
/**
 *******************************************************************************
 * \file iss_sensor_ar0220.c
 *
 *
 *
 *******************************************************************************
*/

/*******************************************************************************
 *  INCLUDE FILES
 *******************************************************************************
 */

#include <iss_sensor_if.h>
#include <iss_sensor_serdes.h>
/**< ISS AEWB plugin is included here to get the default AEWB configuration
          from each sensor */
#include <algorithmLink_issAewb.h>
#include "ar0220_mars_linear_1820x940_setup.h"
#include "ar0220_mars_wdr_1820x940_setup.h"
#include "ar0220_gain_table.h"
#include "ar0220_dcc.h"
#include "ar0220_dcc_1pass.h"

#include <ti/drv/vps/include/devices/bsp_device.h>


/**< UB953 I2C Address */
#define UB953_I2C_ADDRESS           (0x18)
/**< UB953 Alias I2C Address */
#define UB953_I2C_ALIAS_ADDRESS     (0x74)
/**< OV7261 I2C Address */
#define AR0220_I2C_ADDRESS          (0x10)
/**< OV7261 Alias I2C Address */
#define AR0220_I2C_ALIAS_ADDRESS    (0x40)

BspUtils_Ub960I2cParams gAppIssUtilsUb954Cfg_AR0220_CSI2_DesCfg[AR0220_CSI2_DES_CFG_SIZE] = {

    {0x0C, 0x83, 0x10}, // Change to 83 from 0F
    {0x1F, 0x00, 0x10}, /* 1.6Gbps CSI Freq */

    /*CAMERA 1*/
    {0x4C, 0x01, 0x1},        /*Page FPD3 port RX0 registers for R/W*/
    {0x58, 0x5D , 0x1},        /*Enable and set backchannel rate to 50Mbs / I2C passthrough enabled*/
    {0x5C, 0x30, 0x1},        /*set serializer alias to Ox18 (7-bit)*/
    {0x5D, ((uint8_t) (UB953_I2C_ADDRESS << 1U)), 0x0},
    {0x65, ((uint8_t) (UB953_I2C_ALIAS_ADDRESS << 1U)), 0x0},
    {0x5E, ((uint8_t) (AR0220_I2C_ADDRESS << 1U)), 0x0},
    {0x66, ((uint8_t) (AR0220_I2C_ALIAS_ADDRESS << 1U)), 0x0},
    {0x6D, 0x7C, 0x1},        /*CSI and coax mode*/
    {0xD5, 0xF0, 0x1},       /* Set AEQ MIN/MAX widest values*/
    {0x7C, 0x01, 0x1},        /*same as Omnivision*/
    {0x70, 0x2B, 0x1},        /*same as Omnivision*/
    {0x71, 0x2C, 0x1},        /*same as Omnivision*/
    {0x72, 0xE4, 0x1},        /*same as Omnivision*/
	
	{0x32, 0x01, 0x1},        /*CSI0 select*/
    {0x33, 0x03, 0x1},        /*CSI_EN & CSI0 4L*/

    {0x21, 0x03, 0x1},
    {0x20, 0x00, 0x1},

    {0xB9, 0x18, 0x1},        /*ENABLE PARITY ERROR COUNT*/
    {0x42, 0x71, 0x1},        /*ENABLE S-Filter with AEQ*/
    {0x10, 0x59, 0x1},        /*Output Frame on GPIO0 */
};


BspUtils_Ub960I2cParams gUB953_AR0220_CSI2_SerCfg[AR0220_CSI2_SER_CFG_SIZE] = {
	//{0x02, 0x52, 0x10},
	{0x32, 0xE9, 0x10},
	
    {0x06, 0x21, 0x10},
    {0x07, 0x28, 0x10},

    // {0x0E, 0xF0, 0x10},
    // {0x0D, 0x00, 0x10},
    // {0x0D, 0x0C, 0x10},
	
	{0x0E, 0xC0, 0x10},
    {0x0D, 0x0C, 0x10},
    {0x0D, 0x08, 0x10},
	{0x0D, 0x0C, 0x10},
};

/*******************************************************************************
 *  Defines
 *******************************************************************************
 */


#define ISS_SENSORS_AR0220_DEFAULT_EXP_RATIO         (16U)

#define ISS_SENSORS_AR0220_FEATURES         (ISS_SENSOR_FEATURE_LINEAR_MODE|       \
                                             ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE| \
                                             ISS_SENSOR_FEATURE_TWO_PASS_WDR_MODE| \
                                             ISS_SENSOR_FEATURE_MANUAL_EXPOSURE|   \
                                             ISS_SENSOR_FEATURE_MANUAL_GAIN|       \
                                             ISS_SENSOR_FEATURE_DCC_SUPPORTED)


/*******************************************************************************
 *  Data structure's
 *******************************************************************************
 */

struct {

         UInt32                                             maxCoarseIntgTime;
         /**< Max Coarse integration time supported by sensor */
         float                                              lineIntgTimeInMs;
         /**< Line Integration time in ms */
         float                                              pixIntgTimeInMs;
         /**< Pixel Integration time in ms */
} gAr0220DeviceObj;

/*******************************************************************************
 *  Local Functions Declarations
 *******************************************************************************
 */
static Int32 AR0220_Start(UInt32 chId, IssSensor_CreateParams *pCreatePrms);
static Int32 AR0220_MARS_Start(UInt32 chId, IssSensor_CreateParams *pCreatePrms);
static Int32 AR0220_Stop(UInt32 chId, IssSensor_CreateParams *pCreatePrms);
static Int32 AR0220_SetAeParams(IssSensor_CreateParams *pCreatePrms,
         IssSensor_AeParams *pAePrms);
static Int32 AR0220_GetDccParams(IssSensor_CreateParams *pCreatePrms,
         IssSensor_DccParams *pDccPrms);
static AR0220_GetExpParams(IssSensor_CreateParams *pCreatePrms,
         IssSensor_ExposureParams *pExpPrms);
static Void AR0220_InitAewbConfig(
         IssSensor_CreateParams *pCreatePrms,
         AlgorithmLink_IssAewbCreateParams *pAewbPrms);
static Void AR0220_GetIspConfig (
         IssSensor_CreateParams *pCreatePrms,
         IssIspConfigurationParameters *pIspCfg);
static Int32 AR0220_ReadWriteReg (
         IssSensor_CreateParams *pCreatePrms, UInt32 readWriteFlag,
         IssSensor_ReadWriteReg *pReg);

static Int32 AR0220_ReadReg(UInt8   i2cInstId,
                            UInt8 	i2cAddr,
                            const 	UInt16 *regAddr,
                            UInt16 	*regVal,
                            UInt32 	numRegs);
static Int32 AR0220_WriteReg(UInt8                   i2cInstId,
                                                                UInt8                   i2cAddr,
                                                                const UInt16 *regAddr,
                                                                const UInt16 *regVal,
                                                                UInt32                  numRegs);

/*******************************************************************************
 *  Globals
 *******************************************************************************
 */

/* \brief Sensor Params structure used for registering this sensor to chains_iss
 */
static IssSensorIf_Params SensorParams_Ar0220 = {0};

static const UInt8 gAr0220DccCfgLinearAndWdr[] = AR0220DCC_CFG;
static const UInt8 gAr0220DccCfgSinglePassWdr[] = AR0220_1PASSDCC_CFG;

static IssSensor_ConfigRegs gAR0220SensorsLinearConfig[] =
         AR0220_MARS_LINEAR_1820X940_CONFIG;
static IssSensor_ConfigRegs gAR0220SensorsWdrConfig[] =
         AR0220_MARS_WDR_1820X940_CONFIG;
static IssSensor_ConfigRegs gAR0220MarsSensorsLinearConfig[] =
         AR0220_MARS_LINEAR_1820X940_CONFIG;
static IssSensor_ConfigRegs gAR0220MarsSensorsWdrConfig[] =
         AR0220_MARS_WDR_1820X940_CONFIG;

static UInt32 gAr0220GainsTable[ISS_SENSORS_AR0220_ANALOG_MAX_GAIN][3] =
         AR0220_GAIN_TABLE;
/**
 * \brief Structure for ISP H3A AEWB engine parameters.
 */
static vpsissH3aAewbConfig_t aewbConfig_ar0220 =
{
    FALSE, /* enableALowComp */
    FALSE, /* enableMedFilt */
    0xFFF, /* midFiltThreshold */
    {   /* vpsissH3aPaxelConfig_t      winCfg */
        {320u, 280u}, /* Fvid2_PosConfig pos */
        32u,     /* width */
        16u,     /* height */
        20u,     /* horzCount */
        20u,     /* vertCount */
        2,      /* horzIncr */
        2,      /* vertIncr */
    },
    718u,       /* Black Line Vertical Start */
    2u,         /* Black Line Width */
    VPS_ISS_H3A_OUTPUT_MODE_SUM_ONLY, /* vpsissH3aOutputMode_t outMode; */
    2,  /* sumShift */
    1023u, /* satLimit */
    VPS_ISS_H3A_MODE_NORMAL /* vpsissH3aMode_t mode */
} ;

/**
 * \brief Structure for ISP GLBCE WDR Params.
 */
static vpsissGlbceWdrConfig_t glbceWdrCfg_ar0220 =
{
    TRUE,
    {
        0U,24U,64U,114U,172U,237U,307U,383U,464U,549U,638U,731U,828U,928U,
        1031U,1138U,1248U,1361U,1477U,1596U,1717U,1841U,1967U,2096U,2228U,
        2361U,2498U,2636U,2777U,2919U,3064U,3211U,3360U,3511U,3664U,3819U,
        3976U,4134U,4295U,4457U,4622U,4787U,4955U,5125U,5296U,5468U,5643U,
        5819U,5997U,6176U,6357U,6539U,6723U,6908U,7095U,7284U,7474U,7665U,
        7858U,8052U,8248U,8445U,8644U,8843U,9045U,9247U,9451U,9656U,9863U,
        10071U,10280U,10490U,10702U,10915U,11129U,11345U,11561U,11779U,11998U,
        12219U,12440U,12663U,12887U,13112U,13338U,13566U,13794U,14024U,14255U,
        14487U,14720U,14954U,15189U,15426U,15663U,15902U,16142U,16382U,16624U,
        16867U,17111U,17356U,17602U,17849U,18097U,18346U,18596U,18847U,19099U,
        19353U,19607U,19862U,20118U,20375U,20633U,20892U,21152U,21413U,21675U,
        21938U,22202U,22467U,22732U,22999U,23267U,23535U,23805U,24075U,24346U,
        24618U,24891U,25165U,25440U,25716U,25993U,26270U,26549U,26828U,27108U,
        27389U,27671U,27954U,28238U,28522U,28807U,29094U,29381U,29669U,29957U,
        30247U,30537U,30829U,31121U,31414U,31707U,32002U,32297U,32593U,32890U,
        33188U,33487U,33786U,34086U,34387U,34689U,34992U,35295U,35599U,35904U,
        36210U,36516U,36823U,37132U,37440U,37750U,38060U,38371U,38683U,38996U,
        39309U,39623U,39938U,40254U,40570U,40887U,41205U,41523U,41843U,42163U,
        42483U,42805U,43127U,43450U,43774U,44098U,44423U,44749U,45075U,45403U,
        45731U,46059U,46389U,46719U,47049U,47381U,47713U,48046U,48379U,48714U,
        49048U,49384U,49720U,50057U,50395U,50733U,51072U,51412U,51752U,52093U,
        52435U,52777U,53121U,53464U,53809U,54154U,54499U,54846U,55193U,55540U,
        55889U,56238U,56587U,56938U,57289U,57640U,57992U,58345U,58699U,59053U,
        59408U,59763U,60119U,60476U,60833U,61191U,61550U,61909U,62269U,62629U,
        62990U,63352U,63714U,64077U,64441U,64805U,65170U,65535U
    }
};

/*******************************************************************************
 *  Function Definition
 *******************************************************************************
 */

Void IssSensor_AR0220_Init()
{
         Int32 status;
         IssSensorIf_Params *prms = &SensorParams_Ar0220;
         IssSensor_Info *info = &SensorParams_Ar0220.info;

         IssSensorIf_Params_Init(prms);

         strncpy(prms->name, SENSOR_AR0220, ISS_SENSORS_MAX_NAME);

         prms->dccId                  = 143;
         info->width                  = 1820;
         info->height                 = 940;
         info->dataFormat             = SYSTEM_DF_BAYER_BGGR;
         info->bpp                    = SYSTEM_BPP_BITS12;
         info->features               = ISS_SENSORS_AR0220_FEATURES;
         info->aewbMode               = ALGORITHMS_ISS_AEWB_MODE_AEWB;
         info->maxExp                 = 1;
         info->ramOffset              = 24*1024*1024;

         prms->start                  = AR0220_Start;
         prms->stop                   = AR0220_Stop;
         prms->setAeParams            = AR0220_SetAeParams;
         prms->getExpParams           = AR0220_GetExpParams;
         prms->getDccParams           = AR0220_GetDccParams;
         prms->initAewbConfig         = AR0220_InitAewbConfig;
         prms->getIspConfig           = AR0220_GetIspConfig;
         prms->readWriteReg           = AR0220_ReadWriteReg;

         status = IssSensorIf_RegisterSensor(prms);
         UTILS_assert(SYSTEM_LINK_STATUS_SOK == status);
}

#if 1
Void IssSensor_AR0220_MARS_Init()
{
         Int32 status;
         IssSensorIf_Params *prms = &SensorParams_Ar0220;
         IssSensor_Info *info = &SensorParams_Ar0220.info;

         IssSensorIf_Params_Init(prms);

         strncpy(prms->name, SENSOR_AR0220_MARS, ISS_SENSORS_MAX_NAME);

         prms->dccId                  = 143;
         info->width                  = 1820;
         info->height                 = 940;
         info->dataFormat             = SYSTEM_DF_BAYER_BGGR;
         info->bpp                    = SYSTEM_BPP_BITS12;
         info->features               = ISS_SENSORS_AR0220_FEATURES;
         info->aewbMode               = ALGORITHMS_ISS_AEWB_MODE_AEWB;
         info->maxExp                 = 1;
         info->ramOffset              = 24*1024*1024;
                                      
         prms->start                  = AR0220_MARS_Start;
         prms->stop                   = AR0220_Stop;
         prms->setAeParams            = AR0220_SetAeParams;
         prms->getExpParams           = AR0220_GetExpParams;
         prms->getDccParams           = AR0220_GetDccParams;
         prms->initAewbConfig         = AR0220_InitAewbConfig;
         prms->getIspConfig           = AR0220_GetIspConfig;
         prms->readWriteReg           = AR0220_ReadWriteReg;

         status = IssSensorIf_RegisterSensor(prms);
         UTILS_assert(SYSTEM_LINK_STATUS_SOK == status);
}
#endif

/*******************************************************************************
 *  Local Functions Definition
 *******************************************************************************
 */

static Int32 AR0220_Start(UInt32 chId, IssSensor_CreateParams *pCreatePrms)
{
		Vps_printf("Bsp_deviceI2cProbeAll start.......\n");
		Int32 retVal = Bsp_deviceI2cProbeAll(BSP_DEVICE_I2C_INST_ID_4);
		if (retVal == BSP_SOK)
		{
			Vps_printf("Bsp_deviceI2cProbeAll is done...\n");
		}
		else {
			Vps_printf("Bsp_deviceI2cProbeAll is failed...\n");
		}
		
		
         Int32 status = SYSTEM_LINK_STATUS_EFAIL;
         UInt32 cnt, i2cInstId, features, numRegs;
         UInt8 i2cAddr;
         UInt16 regAddr, regValue;
         IssSensor_ConfigRegs *sensorRegs;

         UTILS_assert(NULL != pCreatePrms);

         i2cInstId = pCreatePrms->i2cInstId;
         i2cAddr = pCreatePrms->i2cAddr[chId];
         features = pCreatePrms->enableFeatures[chId];

         if (ISS_SENSOR_FEATURE_LINEAR_MODE ==
                  (features & ISS_SENSOR_FEATURE_LINEAR_MODE))
         {
                  sensorRegs = gAR0220SensorsLinearConfig;
                  numRegs = sizeof(gAR0220SensorsLinearConfig)/
                           sizeof(IssSensor_ConfigRegs);

                  gAr0220DeviceObj.maxCoarseIntgTime = MAX_COARSE_INT_LINES_LINEAR_940P;
         }
         else
         {
                  sensorRegs = gAR0220SensorsWdrConfig;
                  numRegs = sizeof(gAR0220SensorsWdrConfig)/
                           sizeof(IssSensor_ConfigRegs);

                  gAr0220DeviceObj.maxCoarseIntgTime = MAX_COARSE_INT_LINES_WDR_940P;
         }
         gAr0220DeviceObj.lineIntgTimeInMs = 40.9f;
         gAr0220DeviceObj.pixIntgTimeInMs  = 0.032f;

         for (cnt = 0u; cnt < numRegs; cnt ++)
         {
                  regAddr  = sensorRegs[cnt].regAddr;
                  regValue = sensorRegs[cnt].regValue;
                  /* Convert Registers address and value into 8bit array */
                  status = AR0220_WriteReg(i2cInstId, i2cAddr, &regAddr,
                           &regValue, 1u);

                  if (SYSTEM_LINK_STATUS_SOK != status)
                  {
                           Vps_printf("AR0220_Start : status = %x\n", status);
                           Vps_printf("AR0220_WriteReg FAILED : Reg Addr = %x, Reg Val = %x\n", i2cAddr, regAddr);
                           break;
                  }
         }

         return (status);
}

#if 1
static Int32 AR0220_MARS_Start(UInt32 chId, IssSensor_CreateParams *pCreatePrms)
{
         Int32 status = SYSTEM_LINK_STATUS_EFAIL;
         UInt32 cnt, i2cInstId, numRegs, features;
         UInt8 i2cAddr;
         UInt16 regAddr, regValue;
         IssSensor_ConfigRegs *sensorRegs;

         UTILS_assert(NULL != pCreatePrms);

         i2cInstId = pCreatePrms->i2cInstId;
         i2cAddr = pCreatePrms->i2cAddr[chId];
         features = pCreatePrms->enableFeatures[chId];

         if (ISS_SENSOR_FEATURE_LINEAR_MODE ==
                  (features & ISS_SENSOR_FEATURE_LINEAR_MODE))
         {
                  sensorRegs = gAR0220MarsSensorsLinearConfig;
                  numRegs = sizeof(gAR0220MarsSensorsLinearConfig)/
                           sizeof(IssSensor_ConfigRegs);

                  gAr0220DeviceObj.maxCoarseIntgTime = MAX_COARSE_INT_LINES_LINEAR_940P;

         }
         else
         {
                  sensorRegs = gAR0220MarsSensorsWdrConfig;
                  numRegs = sizeof(gAR0220MarsSensorsWdrConfig)/
                           sizeof(IssSensor_ConfigRegs);

                  gAr0220DeviceObj.maxCoarseIntgTime = MAX_COARSE_INT_LINES_WDR_940P;
         }

         gAr0220DeviceObj.lineIntgTimeInMs = 40.9f;
         gAr0220DeviceObj.pixIntgTimeInMs  = 0.032f;

         for (cnt = 0u; cnt < numRegs; cnt ++)
         {
                  regAddr  = sensorRegs[cnt].regAddr;
                  regValue = sensorRegs[cnt].regValue;

                  /* Convert Registers address and value into 8bit array */
                  status = AR0220_WriteReg(i2cInstId, i2cAddr, &regAddr, &regValue, 1u);

                  if (SYSTEM_LINK_STATUS_SOK != status)
                  {
                           Vps_printf("AR0220_WriteReg FAILED : Reg Addr = %x, Reg Val = %x\n", i2cAddr, regAddr);
                           break;
                  }
                  else if(sensorRegs[cnt].delay)
                             BspOsal_sleep(sensorRegs[cnt].delay);
         }

         return (status);
}
#endif

static Int32 AR0220_Stop(UInt32 chId,
         IssSensor_CreateParams *pCreatePrms)
{
         Int32 status = SYSTEM_LINK_STATUS_SOK;
         return (status);
}

static Int32 AR0220_SetAeParams(IssSensor_CreateParams *pCreatePrms,
         IssSensor_AeParams *pAePrms)
{
         Int32 status = SYSTEM_LINK_STATUS_EFAIL;
         UInt32 totalGain, convGain, analogGain, cnt, chId;
         UInt16 regAddr, regValue;
         IssSensor_Info *info = &SensorParams_Ar0220.info;

         UTILS_assert(NULL != pCreatePrms);
         UTILS_assert(NULL != pAePrms);

         if (
               (info->aewbMode == ALGORITHMS_ISS_AEWB_MODE_AE) ||
               (info->aewbMode == ALGORITHMS_ISS_AEWB_MODE_AEWB)
         )
         {
              chId = pAePrms->chId;
              regAddr = 0x3012U;
              regValue = (UInt16) (pAePrms->exposureTime[0] /
                       gAr0220DeviceObj.lineIntgTimeInMs);

              if (regValue > gAr0220DeviceObj.maxCoarseIntgTime)
              {
                       regValue = gAr0220DeviceObj.maxCoarseIntgTime;
              }

              /* Convert Registers address and value into 8bit array */
              status = AR0220_WriteReg(pCreatePrms->i2cInstId,
                       pCreatePrms->i2cAddr[chId], &regAddr, &regValue, 1u);

              if(SYSTEM_LINK_STATUS_SOK == status)
              {
                   totalGain = pAePrms->analogGain;

                   if(totalGain >= 3000U)
                   {
                   	     convGain = 1U;
                   	     analogGain = (UInt32)(totalGain/3);
						/*Enable High Conversion Gain*/
                   }
                   else
                   {
						/*Disable High Conversion Gain*/
                        convGain = 0U;
                        analogGain = (UInt32)(totalGain);
                   }

					/*Write conversion gain to register 0x3362*/
                   {
                             UInt16 conv_gain_t1 = convGain;
                             UInt16 conv_gain_t2 = convGain;
                             UInt16 conv_gain_t3 = convGain;
                             UInt16 conv_gain_t4 = convGain;
                             regAddr = 0x3362U;
                             regValue = (UInt16)
                                            (
                                               (conv_gain_t1) | 
                                               (conv_gain_t2 << 4) |  
                                               (conv_gain_t3 << 8) |  
                                               (conv_gain_t4 << 12)
                                             );
                             status = AR0220_WriteReg(pCreatePrms->i2cInstId,
                                              pCreatePrms->i2cAddr[chId], &regAddr, &regValue, 1u);
                   }


                   /* Sensor support at max 4x analog gain */
                   for (cnt = 0; cnt < ISS_SENSORS_AR0220_ANALOG_MAX_GAIN; cnt ++)
                   {
                            if (analogGain <= gAr0220GainsTable[cnt][0])
                            {
                                     /*Apply same analog gain to all expoures*/
                                     UInt16 analog_gain_t1 = gAr0220GainsTable[cnt][2];
                                     UInt16 analog_gain_t2 = gAr0220GainsTable[cnt][2];
                                     UInt16 analog_gain_t3 = gAr0220GainsTable[cnt][2];
                                     UInt16 analog_gain_t4 = gAr0220GainsTable[cnt][2];
                                     regValue = (UInt16)
                                                        (
                                                            (analog_gain_t1) | 
                                                            (analog_gain_t2 << 4) |  
                                                            (analog_gain_t3 << 8) |  
                                                            (analog_gain_t4 << 12)
                                                        );
                                     break;
                            }
                   }
                   regAddr = 0x3366U;
                   status = AR0220_WriteReg(pCreatePrms->i2cInstId,
                                              pCreatePrms->i2cAddr[chId], &regAddr, &regValue, 1u);


              }
         }

         return (status);
}

static Int32 AR0220_ReadReg(UInt8                   i2cInstId,
                            UInt8                   i2cAddr,
                            const UInt16 *regAddr,
                            UInt16            *regVal,
                            UInt32            numRegs)
{
         Int32  status = SYSTEM_LINK_STATUS_SOK;
         UInt32 cnt;
         UInt8  rawRegVal[4];

         for (cnt = 0; cnt < numRegs; cnt++)
         {
                  /* Convert Registers address and value into 8bit array */
                  rawRegVal[0U] = (UInt8) ((regAddr[cnt] >> 8U) & (UInt8) 0xFF);
                  rawRegVal[1U] = (UInt8) ((regAddr[cnt] >> 0U) & (UInt8) 0xFF);

                  status = Bsp_deviceRawWrite8(i2cInstId, i2cAddr, rawRegVal, 2u);

                  if (FVID2_SOK == status)
                  {
                           status = Bsp_deviceRawRead8(i2cInstId, i2cAddr, &rawRegVal[2U],
                                    2u);

                           regVal[cnt] =
                                    (UInt16)((UInt16)((rawRegVal[2U] & (UInt16) 0xFF) << 8U) |
                                                       ((UInt16)(rawRegVal[3U] & (UInt16) 0xFF)));
                  }
         }

         return (status);
}

static Int32 AR0220_WriteReg(UInt8         i2cInstId,
                             UInt8         i2cAddr,
                             const UInt16 *regAddr,
                             const UInt16 *regValue,
                             UInt32        numRegs)
{
         Int32 status = SYSTEM_LINK_STATUS_EFAIL;
         UInt32 cnt;
         UInt8 rawRegVal[4];

         for (cnt = 0u; cnt < numRegs; cnt ++)
         {
                  /* Convert Registers address and value into 8bit array */
                  rawRegVal[0U] = (UInt8) ((regAddr[cnt] >> 8U) & (UInt8) 0xFF);
                  rawRegVal[1U] = (UInt8) ((regAddr[cnt] >> 0U) & (UInt8) 0xFF);
                  rawRegVal[2U] = (UInt8) ((regValue[cnt] >> 8U) & (UInt8) 0xFF);
                  rawRegVal[3U] = (UInt8) ((regValue[cnt] >> 0U) & (UInt8) 0xFF);

                  status = Bsp_deviceRawWrite8(i2cInstId, i2cAddr, rawRegVal, 4U);

                  if (SYSTEM_LINK_STATUS_SOK != status)
                  {
                           break;
                  }
         }

         return (status);
}

static AR0220_GetDccParams(IssSensor_CreateParams *pCreatePrms,
         IssSensor_DccParams *pDccPrms)
{
         Int32 status = SYSTEM_LINK_STATUS_SOK;
         UInt32 features;

         UTILS_assert(NULL != pCreatePrms);
         UTILS_assert(NULL != pDccPrms);

         features = pCreatePrms->enableFeatures[pDccPrms->chId];

         if (ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE ==
                  (features & ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE))
         {
                  pDccPrms->pDccCfg = (UInt8 *)gAr0220DccCfgSinglePassWdr;
                  pDccPrms->dccCfgSize = sizeof(gAr0220DccCfgSinglePassWdr);
         }
         else
         {
                  pDccPrms->pDccCfg = (UInt8 *)gAr0220DccCfgLinearAndWdr;
                  pDccPrms->dccCfgSize = sizeof(gAr0220DccCfgLinearAndWdr);
         }

         return (status);
}

static AR0220_GetExpParams(IssSensor_CreateParams *pCreatePrms,
         IssSensor_ExposureParams *pExpPrms)
{
    Int32 status = SYSTEM_LINK_STATUS_SOK;

    UTILS_assert(NULL != pCreatePrms);
    UTILS_assert(NULL != pExpPrms);

    {
        pExpPrms->expRatio = ISS_SENSORS_AR0220_DEFAULT_EXP_RATIO;
    }

    return (status);
}

static Void AR0220_InitAewbConfig(
         IssSensor_CreateParams *pCreatePrms,
         AlgorithmLink_IssAewbCreateParams *pAewbPrms)
{
         AlgorithmLink_IssAewbAeDynamicParams *dynPrms = NULL;

         UTILS_assert(NULL != pCreatePrms);
         UTILS_assert(NULL != pAewbPrms);

         /* Set H3A Params */
         pAewbPrms->h3aParams.winCountH = aewbConfig_ar0220.winCfg.horzCount;
         pAewbPrms->h3aParams.winCountV = aewbConfig_ar0220.winCfg.vertCount;
         pAewbPrms->h3aParams.winSizeH  = aewbConfig_ar0220.winCfg.width;
         pAewbPrms->h3aParams.winSizeV  = aewbConfig_ar0220.winCfg.height;
         pAewbPrms->h3aParams.winSkipH  = aewbConfig_ar0220.winCfg.horzIncr;
         pAewbPrms->h3aParams.winSkipV  = aewbConfig_ar0220.winCfg.vertIncr;
         pAewbPrms->numH3aPlanes = 1u;

         pAewbPrms->dataFormat = SensorParams_Ar0220.info.dataFormat;
         pAewbPrms->mode = (AlgorithmLink_IssAewbMode)SensorParams_Ar0220.info.aewbMode;

         if (ISS_SENSOR_FEATURE_LINEAR_MODE ==
                           (ISS_SENSOR_FEATURE_LINEAR_MODE&pCreatePrms->enableFeatures[0]))
         {
                  pAewbPrms->isWdrEnable = FALSE;
         }
         else
         {
                  pAewbPrms->isWdrEnable = TRUE;
         }

         pAewbPrms->dccCameraId = SensorParams_Ar0220.dccId;

         dynPrms = &pAewbPrms->aeDynParams;

         if (ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE ==
                           (ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE &
                                    pCreatePrms->enableFeatures[pAewbPrms->chId]))
         {
                  dynPrms->targetBrightnessRange.min = 35;
                  dynPrms->targetBrightnessRange.max = 45;
                  dynPrms->targetBrightness = 40;
         }
         else
         {
                  dynPrms->targetBrightnessRange.min = 35;
                  dynPrms->targetBrightnessRange.max = 45;
                  dynPrms->targetBrightness = 40;
         }
         dynPrms->threshold = 5;

         dynPrms->exposureTimeRange[0].min = 100;
         dynPrms->exposureTimeRange[0].max = 33333;
         dynPrms->apertureLevelRange[0].min = 1;
         dynPrms->apertureLevelRange[0].max = 1;
         dynPrms->sensorGainRange[0].min = 1000;
         dynPrms->sensorGainRange[0].max = 12000;
         dynPrms->ipipeGainRange[0].min = 512;
         dynPrms->ipipeGainRange[0].max = 512;

         dynPrms->numAeDynParams = 1;
         dynPrms->exposureTimeStepSize = 1;
         dynPrms->enableBlc = TRUE;

         pAewbPrms->calbData = NULL;
}

static Void AR0220_GetIspConfig (
         IssSensor_CreateParams *pCreatePrms,
         IssIspConfigurationParameters *pIspCfg)
{
         UInt32 chId;

         UTILS_assert(NULL != pIspCfg);
         UTILS_assert(NULL != pCreatePrms);

         /* Assuming Channel Id as 0 */
         chId = 0;

         /* Override common settings for specific sensor */
         pIspCfg->aewbCfg = &aewbConfig_ar0220;

         if (ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE ==
                           (ISS_SENSOR_FEATURE_ONE_PASS_WDR_MODE &
                                    pCreatePrms->enableFeatures[chId]))
         {
                  /* WDR Config */
                  pIspCfg->glbceWdrCfg = &glbceWdrCfg_ar0220;
         }
}

static Int32 AR0220_ReadWriteReg (
         IssSensor_CreateParams *pCreatePrms, UInt32 readWriteFlag,
         IssSensor_ReadWriteReg *pReg)
{
         Int32 status;
         UInt32 chId;

         UTILS_assert(NULL != pReg);
         UTILS_assert(NULL != pReg);

         /* Assuming Channel Id as 0 */
         chId = pReg->chId;

         if (readWriteFlag)
         {
                  status = AR0220_WriteReg(pCreatePrms->i2cInstId,
                           pCreatePrms->i2cAddr[chId], &pReg->regAddr, &pReg->regValue, 1u);
         }
         else
         {
                  status = AR0220_ReadReg(pCreatePrms->i2cInstId,
                           pCreatePrms->i2cAddr[chId], &pReg->regAddr, &pReg->regValue, 1u);
         }

         return (status);
}
